*********************************************************
* KSSsample.sas
* 
* Updated July 24, 2007
* Claudia R. Sahm (Email: Claudia.R.Sahm@frb.gov)
* 
* Creates data set for "Imputing Risk Tolerance from
* Survey Responses" with Miles Kimball (mkimball@umich.edu)
* and Matthew Shapiro (shapiro@umich.edu) forthcoming in 
* Journal of the American Statistical Association
*********************************************************;

%let home = G:\res\Data\HRS\;

** HRS Public-Access Data Files;
libname hrs1992 "&home.1992";
libname hrs1994 "&home.1994";
libname hrs1996 "&home.1996";
libname hrs1998 "&home.1998";
libname hrs2000 "&home.2000";
libname hrs2002 "&home.2002";
libname tracker "&home.tracker";

** HRS-Rand Data Set (Version G);
libname rand "&home.Rand";
	%let fmtlib=&home.Rand\Fmt\;  
libname library "&fmtlib";

%let maclib=&home.\Rand\Mac\; 
	%include "&maclib.wvlist.mac";
%let maxwv = 6;

libname final  'G:\res\Research\RiskPrefs\ImputeRisk_KSS';

********************* HRS 1992 ****************;
data risk92(keep=hhid pn southov otherov risk4_1992);
set hrs1992.HEALTH;

if v5122=5 and v5123=5 then risk4_1992 = 1;
	else if v5122=5 and v5123=1 then risk4_1992 = 2;
	else if v5122=1 and v5123=5 then risk4_1992 = 3;
	else if v5122=1 and v5123=1 then risk4_1992 = 4;
	else risk4_1992 = 0;

/* NOTE: The following sampling variables are used for a robustness check
   The main estimation sets equal weight to all individuals
   But the HRS does oversample blacks, Hispanics, and residents of Florida */

** Reside in South-Atlantic Division (includes Florida) and in Oversample;
if v53 = 5 and v11 ne 1 then southov = 1;
	else southov = 0;

** Black -- only to define Other Oversampling Group;
if v48 = 2 or v68 = 2 then black = 1;
	else black = 0;

** Hispanic -- only to define Other Oversampling Group;
if v48 = 5 or v68 = 5 then hispanic = 1;
	else hispanic = 0;

** Non-South-Atlantic, non-black, non-hispanic, but in Oversample group;
if southov = 0 and black = 0  and hispanic = 0 and v11 ne 1 then otherov = 1;
	else otherov = 0; 

****************** HRS 1994 ************************;
data risk94(keep=hhid pn risk6_1994);
	set hrs1994.W2MOD5(keep=hhid pn W9344-W9348); 

if W9344=5 and W9346 = 5 and W9347 = 5 then risk6_1994=1;
	else if W9344=5 and W9346 = 5 and W9347 = 1 then risk6_1994=2;
	else if W9344=5 and W9346 = 1 then risk6_1994=3;
	else if W9344=1 and W9345 = 5 then risk6_1994=4;
	else if W9344=1 and W9345 = 1 and W9348 = 5 then risk6_1994=5;
	else if W9344=1 and W9345 = 1 and W9348 = 1 then risk6_1994=6;
	else risk6_1994 = 0;

****************** HRS 1998 ************************;
data risk98(keep=hhid pn risk6_1998);
	set hrs1998.H98H_R(keep=hhid pn F4614-F4618);

if  F4614=1 and  F4617 = 1 and  F4618 = 1 then risk6_1998=1;
	else if F4614=1 and F4617 = 1 and F4618 = 2 then risk6_1998=2;
	else if F4614=1 and F4617 = 2 then risk6_1998=3;
	else if F4614=2 and F4615 = 1 then risk6_1998=4;
	else if F4614=2 and F4615 = 2 and F4616 = 1 then risk6_1998=5;
	else if F4614=2 and F4615 = 2 and F4616 = 2 then risk6_1998=6;
	else risk6_1998 = 0;

****************** HRS 2000 ************************;
data risk00(keep=hhid pn  
	risk6_2000);
	set hrs2000.H00H_R(keep=hhid pn G5027 G5033-G5036);

if  G5027=1 and G5035 = 1 and  G5036 = 1 then risk6_2000=1; 
	else if G5027=1 and G5035 = 1 and G5036 = 2 then risk6_2000=2;
	else if G5027=1 and G5035 = 2 then risk6_2000=3;
	else if G5027=2 and G5033 = 1 then risk6_2000=4;
	else if G5027=2 and G5033 = 2 and G5034 = 1 then risk6_2000=5;
	else if G5027=2 and G5033 = 2 and G5034 = 2 then risk6_2000=6;
	else risk6_2000 = 0;

****************** HRS 2002 ************************;
data risk02(keep=hhid pn risk6_2002);
	set hrs2002.H02P_R(keep=hhid pn HP036-HP040 HP060-HP064 HP065-HP069);

if  HP036=1 and HP039 = 1 and  HP040 = 1 then risk6_2002=1; 
	else if HP036=1 and HP039 = 1 and HP040 = 2 then risk6_2002=2;
	else if HP036=1 and HP039 = 2 then risk6_2002=3;
	else if HP036=2 and HP037 = 1 then risk6_2002=4;
	else if HP036=2 and HP037 = 2 and HP038 = 1 then risk6_2002=5;
	else if HP036=2 and HP037 = 2 and HP038 = 2 then risk6_2002=6;
	else risk6_2002 = 0;

************* 2004 Tracker (Version 2.0) **************;
data tracker;
	set tracker.trk2004(keep=hhid pn 
	gender birthyr birthmo schlyrs firstiw study
	wtcohort afinr 
	aiwyear ciwyear eiwyear fiwyear giwyear hiwyear
	aiwmonth ciwmonth eiwmonth fiwmonth giwmonth hiwmonth
	awgtr cwgtr ewgtr fwgtr gwgtr hwgtr
	acouple ccouple ecouple fcouple gcouple hcouple);

** HRS cohorts - excludes AHEAD overlap households;
if study = 1;

** Original respondents;
if firstiw = 1;

** Impute June for missing birth months -- (14 cases in 12,521 respondents);
if birthmo <= 0 then birthmo = 6;
 
** Impute mean birthyr of group to one individual (in gamble sample) with mising year;
if birthyr = 0 and wtcohort = 36 then birthyr = 1934;

** Gender;
if gender = 1 then male = 1;
	else if gender = 2 then male = 0;
	drop gender;

** Coupled or Partnered;
if acouple = 1 then couple_1992 = 1;
	else if acouple = 5 then couple_1992 = 0;
if ccouple = 1 then couple_1994 = 1;
	else if ccouple = 5 then couple_1994 = 0;
if fcouple = 1 then couple_1998 = 1;
	else if fcouple = 5 then couple_1998 = 0;
if gcouple = 1 then couple_2000 = 1;
	else if gcouple = 5 then couple_2000 = 0;
if hcouple = 1 then couple_2002 = 1;
	else if hcouple = 5 then couple_2002 = 0;

** Financial Respondent in 1992;
if afinr = 1 then finr = 1;
	else finr = 0;

proc sort data = tracker; by hhid pn;
proc sort data = risk92; by hhid pn;
proc sort data = risk94; by hhid pn;
proc sort data = risk98; by hhid pn;
proc sort data = risk00; by hhid pn;
proc sort data = risk02; by hhid pn;

** Merge HRS Gambles and Tracker;
data hrs9202;
	merge tracker(in=inT) 
		risk92 risk94 risk98 risk00 risk02;
	by hhid pn;
	if inT;

** Missing values to zeros;
if risk4_1992 = . then risk4_1992 = 0;
if risk6_1994 = . then risk6_1994 = 0;
if risk6_1998 = . then risk6_1998 = 0;
if risk6_2000 = . then risk6_2000 = 0;
if risk6_2002 = . then risk6_2002 = 0;

** ID to match with RAND-HRS file;
hhidpn = hhid*1000 + pn;

proc sort data = hrs9202; by hhidpn;

************* RAND HRS (Version F) **************;

** Macro for Income and Wealth Variables **;
%macro wealth(n,yr);
/* NOTE: Rand-HRS file includes imputations for wealth and income */

** Household financial assets, excluding IRAs (not net value) in 2002 dollars;
if H&n.ASTCK >= 0 and H&n.ACD >= 0 and  H&n.ACHCK >= 0 and  H&n.ABOND >= 0 and H&n.AOTHR >= 0 then 
	fwealth_&yr = (H&n.ASTCK + H&n.ACD + H&n.ACHCK + H&n.ABOND + H&n.AOTHR )* pindex(2002)/pindex(&yr);
	else fwealth_&yr = 0;

** Log of total financial assets;
if fwealth_&yr > 0 then lfwealth_&yr = log(fwealth_&yr);
	else lfwealth_&yr = 0;

** Indicator of non-positive total financial assets;
if lfwealth_&yr = 0 then zfwealth_&yr = 1;
	else zfwealth_&yr = 0;

** Log of total household income over previous year in 2002 dollars; 
%let yrL = %eval(&yr-1);
if H&n.ITOT > 0 then ltotinc_&yr = log(H&n.ITOT * pindex(2002)/pindex(&yrL));
	else ltotinc_&yr = 0;

** Indicator for zero household income;
if H&n.ITOT = 0 then ztotinc_&yr = 1;
	else ztotinc_&yr = 0;

** Share of financial assets held directly in stocks;
if H&n.ASTCK >= 0  and fwealth_&yr > 0
	then stocksh_&yr = (H&n.ASTCK*pindex(2002)/pindex(&yr)) / fwealth_&yr;
	else stocksh_&yr = 0;
%mend;

** Macro for Demographic Variables;
%macro demog(n,yr);
/* NOTE: Spouse is specific to a wave*/

** Black: respondent or spouse;
if RARACEM = 2 or S&n.RACEM = 2 then black_&yr = 1;
	else black_&yr = 0;

** Hispanic: respondent or spouse;
if RAHISPAN = 1 or S&n.HISPAN = 1 then hispanic_&yr = 1;
	else hispanic_&yr = 0;

** Indicator for spouse in 1931 to 1941 birth cohorts;
if 1931 <= S&n.BYEAR <= 1941 then sbc3141_&yr = 1;
	else sbc3141_&yr = 0;

** Indicator for spouse in 1942 to 1980 birth cohorts;
if S&n.BYEAR >= 1942 then sbc4280_&yr = 1;
	else sbc4280_&yr = 0;

** Birth year of spouse;
if S&n.BYEAR > 0 then sbirthyr_&yr = S&n.BYEAR;
	else sbirthyr_&yr = 0;

** Census Region: 1=Northeast, 2=Midwest, 3=South, 4=West;
if R&n.CENREG = 1 then east_&yr = 1;
	else east_&yr = 0;
if R&n.CENREG = 3 then south_&yr = 1;
	else south_&yr = 0;
if R&n.CENREG = 4 then west_&yr = 1;
	else west_&yr = 0;
%mend;

data rand;
	set rand.rndhrs_f(keep = HHIDPN	RAHISPAN RARACEM
	%wvlist(R,CENREG) 
	%wvlist(S,RACEM HISPAN BYEAR) 
 	%wvlist(H,ITOT ASTCK ACHCK ACD ABOND AOTHR AIRA) );
FORMAT _ALL_ ; /* Don't use rand formats*/

/* NOTE: CPI-U: http://data.bls.gov/
   Not Seasonally Adjusted, U.S. city average, 
   All items, 1982-84=100, CUUR0000SA0,CUUS0000SA0 */

array pindex{1991:2002} _temporary_;
  pindex{1991} = 136.2 ; 
  pindex{1992} = 140.3 ;
  pindex{1993} = 144.5 ; 
  pindex{1994} = 148.2 ;  
  pindex{1995} = 152.4 ;
  pindex{1996} = 156.9 ; 
  pindex{1997} = 160.5 ;  
  pindex{1998} = 163.0 ;
  pindex{1999} = 166.6 ; 
  pindex{2000} = 172.2 ;  
  pindex{2001} = 177.1 ;
  pindex{2002} = 179.9 ; 

%wealth(1,1992); %wealth(2,1994); %wealth(3,1996);
%wealth(4,1998); %wealth(5,2000); %wealth(6,2002);

%demog(1,1992); %demog(2,1994); %demog(3,1996);
%demog(4,1998); %demog(5,2000); %demog(6,2002);

** Merge HRS and RAND HRS data sets --- Sample Selection;

data KSSsample(keep=hhidpn birthyr male 
	bc3141 bc4280 finr southov otherov schlyrs dropout higrad smcoll pscoll

	iwyr_1992 iwyr_1994 iwyr_1998 iwyr_2000 iwyr_2002
	iwmo_1992 iwmo_1994 iwmo_1998 iwmo_2000 iwmo_2002
	wgt_1992 wgt_1994 wgt_1998 wgt_2000 wgt_2002

	couple_1992 couple_1994 couple_1998 couple_2000 couple_2002

	sbc3141_1992 sbc3141_1994 sbc3141_1998 sbc3141_2000 sbc3141_2002
	sbc4280_1992 sbc4280_1994 sbc4280_1998 sbc4280_2000 sbc4280_2002
	age_1992 age_1994 age_1998 age_2000 age_2002
	sage_1992 sage_1994 sage_1998 sage_2000 sage_2002

	east_1992 east_1994 east_1998 east_2000 east_2002
	west_1992 west_1994 west_1998 west_2000 west_2002
	south_1992 south_1994 south_1998 south_2000 south_2002

	black_1992 black_1994 black_1998 black_2000 black_2002
	hispanic_1992 hispanic_1994 hispanic_1998 hispanic_2000 hispanic_2002

	c92 c94 c98 c00 c02 ngambles original biasfree

	ltotinc_1992 ltotinc_1994 ltotinc_1998 ltotinc_2000 ltotinc_2002
	ztotinc_1992 ztotinc_1994 ztotinc_1998 ztotinc_2000 ztotinc_2002

	lfwealth_1992 lfwealth_1994 lfwealth_1998 lfwealth_2000 lfwealth_2002
	zfwealth_1992 zfwealth_1994 zfwealth_1998 zfwealth_2000 zfwealth_2002

	stocksh_1992 fwealth_1992);
	merge hrs9202(in=inHRS) rand(in=inRD);
	by hhidpn;

if inHRS = 1; 
if inRD = 1;

* Must have answered at least one gamble in 1992 or 1994 HRS;
if risk4_1992 > 0 or risk6_1994 > 0;

** Indicator for 1931 to 1941 birth cohorts;
if 1931 <= birthyr <= 1941 then bc3141 = 1;
	else bc3141 = 0;

** Indicator for 1942 to 1980 birth cohorts;
if 1942 <= birthyr then bc4280 = 1;
	else bc4280 = 0;

** Initialize gamble response categories;
c92 = 0; c94 = 0; c98 = 0; c00 = 0; c02 = 0;

** Initialize interview year;
iwyr_1992 = 0; iwyr_1994 = 0; iwyr_1998 = 0;  
iwyr_2000 = 0; iwyr_2002 = 0; 

** Initialize interview month;
iwmo_1992 = 0; iwmo_1994 = 0; iwmo_1998 = 0;  
iwmo_2000 = 0; iwmo_2002 = 0; 

** Initialize respondent sample weight;
wgt_1992 = 0; wgt_1994 = 0; wgt_1998 = 0;  
wgt_2000 = 0; wgt_2002 = 0; 

** Initialize respondent age;
age_1992 = 0; age_1994 = 0; age_1998 = 0;  
age_2000 = 0; age_2002 = 0; 

** Initialize spouse's age;
sage_1992 = 0; sage_1994 = 0; sage_1998 = 0;  
sage_2000 = 0; sage_2002 = 0; 

** Core Interiew Obtained;
if aiwyear > 0 then do;
	if awgtr > 0 then wgt_1992 = awgtr;
	iwyr_1992 = aiwyear;
	iwmo_1992 = aiwmonth;
	age_1992 = (iwyr_1992 - birthyr) + (iwmo_1992 - birthmo)/12;
	if sbirthyr_1992 > 0 then sage_1992 = (iwyr_1992 - sbirthyr_1992);
	if risk4_1992 > 0 then c92 = risk4_1992;
end;

if ciwyear > 0 then do;
	if cwgtr > 0 then wgt_1994 = cwgtr;
	iwyr_1994 = ciwyear;
	iwmo_1994 = ciwmonth;
	age_1994 = (iwyr_1994 - birthyr) + (iwmo_1994 - birthmo)/12;
	if sbirthyr_1994 > 0 then sage_1994 = (iwyr_1994 - sbirthyr_1994);
	if risk6_1994 > 0 then c94 = risk6_1994;
end;

if fiwyear > 0 then do;
	if fwgtr > 0 then wgt_1998 = fwgtr;
	iwyr_1998 = fiwyear;
	iwmo_1998 = fiwmonth;
	age_1998 = (iwyr_1998 - birthyr) + (iwmo_1998 - birthmo)/12;
	if sbirthyr_1998 > 0 then sage_1998 = (iwyr_1998 - sbirthyr_1998);
	if risk6_1998 > 0 then c98 = risk6_1998;
end;

if giwyear > 0  then do;
	if gwgtr > 0 then wgt_2000 = gwgtr;
	iwyr_2000 = giwyear;
	iwmo_2000 = giwmonth;
	age_2000 = (iwyr_2000 - birthyr) + (iwmo_2000 - birthmo)/12;
	if sbirthyr_2000 > 0 then  sage_2000 = (iwyr_2000 - sbirthyr_2000);
	if risk6_2000 > 0 then c00 = risk6_2000;
end;

if hiwyear > 0 then do;
	if hwgtr > 0 then wgt_2002 = hwgtr;
	iwyr_2002 = hiwyear;
	iwmo_2002 = hiwmonth;
	age_2002 = (iwyr_2002 - birthyr) + (iwmo_2002 - birthmo)/12;
	if sbirthyr_2002 > 0 then sage_2002 = (iwyr_2002 - sbirthyr_2002);
	if risk6_2002 > 0 then c02 = risk6_2002;
end;

** Missing values to zero;
if couple_1992 = . then couple_1992 = 0;
if couple_1994 = . then couple_1994 = 0;
if couple_1998 = . then couple_1998 = 0;
if couple_2000 = . then couple_2000 = 0;
if couple_2002 = . then couple_2002 = 0;

** Number of gamble responses across 1992-2002 HRS;
ngambles = 0;
	if c92 > 0 then ngambles = ngambles + 1;
	if c94 > 0 then ngambles = ngambles + 1;
	if c98 > 0 then ngambles = ngambles + 1;
	if c00 > 0 then ngambles = ngambles + 1;
	if c02 > 0 then ngambles = ngambles + 1;

** Number of original version gamble responses 1992, 1994 HRS;
original = 0;
	if c92 > 0 then original = original + 1;
	if c94 > 0 then original = original + 1;

** Number of revised version gamble responses 1998, 2000, 2002 HRS;
biasfree = 0;
	if c98 > 0 then biasfree = biasfree + 1;
	if c00 > 0 then biasfree = biasfree + 1;
	if c02 > 0 then biasfree = biasfree + 1;

** 4-way groups for years of schooling;
if schlyrs < 12 then dropout = 1;
	else dropout = 0;
if schlyrs = 12 then higrad = 1;
	else higrad = 0;
if schlyrs > 12 & schlyrs <= 16 then smcoll = 1;
	else smcoll = 0;
if schlyrs = 17 then pscoll = 1;
	else pscoll = 0;

proc sort data = KSSsample;
by hhidpn;

proc means data = KSSsample;
title "Sample for KSS"; run;

data final.KSSsample;
	set KSSsample;

** Output to Excel for Transfer to Matlab **;
data KSSM;
	set KSSsample; if male = 1;
data KSSW;
	set KSSsample; if male = 0;

** Macro Exports to Excel **;
%macro outts(name);
data &name; 
	set &name;
proc dbload dbms=excel data=&name;
path="G:\res\Research\RiskPrefs\ImputeRisk_KSS\&name..xls";
putnames yes;
limit=0;
load;
run;
%mend outts;
%outts(KSSM);
%outts(KSSW);

** Table 2 Distribution of Risk Tolerance Responses;
proc freq data = KSSsample;
	tables c92 c94 c98 c00 c02;

** Summary Statistics for Gamble Respondents;
proc means data = KSSsample;
	var black_1992 hispanic_1992 male age_1992 ltotinc_1992 ztotinc_1992 lfwealth_1992 zfwealth_1992;
	where c92 > 0;
 
proc means data = KSSsample;
	var black_1994 hispanic_1994 male age_1994 ltotinc_1994 ztotinc_1994 lfwealth_1994 zfwealth_1994;
	where c94 > 0;

proc means data = KSSsample;
	var black_1998 hispanic_1998 male age_1998 ltotinc_1998 ztotinc_1998 lfwealth_1998 zfwealth_1998;
	where c98 > 0;

proc means data = KSSsample;
	var black_2000 hispanic_2000 male age_2000 ltotinc_2000 ztotinc_2000 lfwealth_2000 zfwealth_2000;
	where c00 > 0;

proc means data = KSSsample;
	var black_2002 hispanic_2002 male age_2002 ltotinc_2002 ztotinc_2002 lfwealth_2002 zfwealth_2002; 
	where c02 > 0;

proc freq data = KSSsample;
tables ngambles original biasfree original*biasfree;

** Correlation Between 1992 and 1994 Gamble Responses **;
data gambles9294;
	set KSSsample; 

	if c92 > 0 and c94 > 0;

** Four-way gamble categories for 1994 HRS;
if c94 in(1,2) then c94_4 = 1;
	else if c94 = 3 then c94_4 = 2;
	else if c94 = 4 then c94_4 = 3;
	else if c94 in(5,6) then c94_4 = 4;

proc corr data = gambles9294 PEARSON SPEARMAN KENDALL; 
	var c92 c94_4;
run;
